#!/usr/bin/perl

## Date and Version
my $version= '1.1';
my $version_date = '19_june_2011';

##################################################
## Documentation POD
##################################################

##################################################
## Modules
##################################################
## Perl modules
use strict;
use warnings;
use diagnostics;

use Time::Seconds;
use Getopt::Long;
use File::Basename;


##################################################
## Globals variables
##################################################

# Initializations
my $Start_time_hr = humanReadableDate();
my ($Delay, $Monitoring_interval, $Inputs, $Outputs) = (0, 600, '', '');
my $DEBUG = 0;


##################################################
## Manage options
##################################################

# Get options from command line
GetOptions ('delay=i'		=> \$Delay,
			'interval=i'	=> \$Monitoring_interval,
			'inputs=s'		=> \$Inputs,
			'outputs=s'		=> \$Outputs);


##################################################
## Main code
##################################################

# Welcome message
print "##############################################################\n";
print "#         Welcome in Workflow blocker (Version $version)          #\n"; 
print "##############################################################\n";
print "Start time: " . $Start_time_hr . "\n\n";

# Initializations
my $Already_spent = 0;
my $Delay_object = Time::Seconds->new($Delay);
my $Monitoring_interval_object = Time::Seconds->new($Monitoring_interval);

# Verbose
print 'Workflow/Tool execution will be locked for around ' . $Delay_object->pretty() . ' (+/- monitoring interval)' . "\n\n";

print 'The monitoring interval has been set to ' . $Monitoring_interval_object->pretty() . "\n\n";

my @All_inputs = split(':', $Inputs);
my @All_outputs = split(':', $Outputs);

for (my $i = 0; $i < scalar(@All_inputs); $i++) {
	print 'Creation of a symlink for file: ' . basename($All_inputs[$i]) . "\n\n" if ($DEBUG);
	symlink($All_inputs[$i], $All_outputs[$i]);
}

print 'Beginning of the waiting period:' . "\n\n";

while ($Already_spent < $Delay) {
	my $Already_spent_object = Time::Seconds->new($Already_spent);
	print '   Execution have already been delayed for ' . $Already_spent_object->pretty() . "\n";
	
	sleep $Monitoring_interval;
	$Already_spent += $Monitoring_interval;
}

print "\n" . 'Workflow/Tool execution will now be unlocked, thanks for waiting !' . "\n\n";

print "Stop time: " . humanReadableDate() . "\n";
print "##############################################################\n";
print "#                      End of execution                      #\n"; 
print "##############################################################\n";

##################################################
## Basic common functions
##################################################

sub humanReadableDate {
	
	# Recovers parameters
	my $time = shift || time;
	
	# Split time string
	my ($seconde, $minute, $heure, $jour, $mois, $annee, $jour_semaine, $jour_annee, $heure_hiver_ou_ete) = localtime($time);
	$mois  += 1;
	$annee += 1900;

	# On rajoute 0 si le chiffre est compris entre 1 et 9
	foreach ( $seconde, $minute, $heure, $jour, $mois, $annee ) { s/^(\d)$/0$1/; }
	
	return "$jour-$mois-$annee" . '_' . "$heure:$minute:$seconde";
}
