##################################################
## Modules
##################################################

from time import localtime, strftime
import optparse, sys, logging, os, subprocess, fnmatch, re

# store true : argument without value e.g action='store_true'

##################################################
## Functions
##################################################

def log(s):

	if(useLog):
		print s
		logging.debug("  "+s)
	else:
		print s

def initLog(p):

	LOG_FILENAME = p
	logging.basicConfig(filename=LOG_FILENAME,level=logging.DEBUG)
	logging.debug('Logger init')

def locate(pattern, root=os.curdir):
	'''Locate all files matching supplied filename pattern in and below
	supplied root directory.'''
	for path, dirs, files in os.walk(os.path.abspath(root)):
		for filename in fnmatch.filter(files, pattern):
			yield os.path.join(path, filename)

def getListFromFile (list_of_reads_file):
	# This function reads an input file that contains the complete path of a reads file on each line.
	# A list of reads file is built during the process
	
	# Initializations
	reads_list = []
	
	# Build list
	try:
		read_file_obj = open(list_of_reads_file, 'r')
	except:
		print 'Error: Cannot open/read file: ', list_of_reads_file
	
	for line in read_file_obj.readlines():
		reads_list.append(line.strip())
		
	read_file_obj.close()
	
	return reads_list
	
def check_output(*popenargs, **kwargs):
	# Backported from Python 2.7 as it's implemented as pure python on stdlib.
	# Usage example: check_output(['/usr/bin/python', '--version'])

	process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
	output, unused_err = process.communicate()
	retcode = process.poll()
	
	if retcode:
		cmd = kwargs.get("args")
		if cmd is None:
			cmd = popenargs[0]
			error = subprocess.CalledProcessError(retcode, cmd)
			error.output = output
			raise error
		
	return output

##################################################
## Main code
##################################################

if __name__=="__main__":
	
	# Initializations
	version = "1.1"
	cmd_args_array = []
	cmd_args = ""
	logPass = "output.log"	
	useLog = False

	# Welcome message
	print("################################################################")
	print("#   Welcome in Abyss assembler Galaxy Wrapper  (Version " + version + ")   #")
	print("################################################################")
	print "Start time: ", strftime("%d-%m-%Y_%H:%M:%S", localtime()), "\n"

	print("Parameters recovery:\n")

	# Recovers parameters
	parser = optparse.OptionParser("usage: %prog [options] arg1 arg2")

	parser.add_option("-L","--LogC", "--logContainer", dest="logC" , type="string")

	parser.add_option("-k","--kmer", dest="kmer")
	parser.add_option("-r","--reads", dest="list_of_file")
	parser.add_option("-o","--out", dest="out")
	
	parser.add_option("--chastity", dest="chastity",  type="string")       # yes or no
	parser.add_option("--trim-masked", dest="trim_masked",  type="string") # yes or no
	parser.add_option("--quality", dest="quality",  type="string")         # standard or illumina
	
	parser.add_option("--use-bubble", dest="use_bubble",type="string") # yes or no
	
	parser.add_option("--ch", dest="cov_hist")
	parser.add_option("-s","--snp", dest="snp")
	parser.add_option("-g","--graph", dest="graph")
	
	parser.add_option("-b","--bubbles", dest="bubbles")
	parser.add_option("-E","--erode-strand", dest="erode_strand")
	parser.add_option("-e","--erode", dest="erode")
	
	parser.add_option("-q","--trim_quality", dest="trim_quality")
	parser.add_option("-t","--trim-length", dest="trim_length",type="string")
	parser.add_option("-c","--coverage", dest="coverage")
	
	(options, args) = parser.parse_args()

	# Command line building and parameters simple check
	cmd_args_array.append("ABYSS")
	
	if options.logC:
		initLog(options.logC)
		logPass = options.logC
		useLog = True
		
	cmd_args_array.append("-v")

	if options.kmer:
		log("kmer size: "+options.kmer)
		cmd_args_array.append("-k")
		cmd_args_array.append(options.kmer)
	else:
		print "Error: no k-mer size specified"
		sys.exit()

	if options.out:
		log("Output File: "+options.out)
		cmd_args_array.append("-o")
		cmd_args_array.append(options.out)
	else:
		print "Error: No output file name specified"
		sys.exit()
		
	if options.use_bubble:
		log("Use bubble: "+options.use_bubble)
		
		if (options.use_bubble == "no"):
			cmd_args_array.append("--no-bubbles")
		else:
				if options.bubbles:
					log("Bubbles: "+options.bubbles)
					
					if(options.bubbles != "undef"):
						cmd_args_array.append("-b")
						cmd_args_array.append(options.bubbles)
		
	if options.chastity:
		log("Chastity: "+options.chastity)
		
		if (options.chastity == "yes"):
			cmd_args_array.append("--chastity")
		else:
			cmd_args_array.append("--no-chastity")
			
	if options.quality:
		log("Quality: "+options.quality)
		
		if (options.quality == "standard"):
			cmd_args_array.append("--standard-quality")
		else:
			cmd_args_array.append("--illumina-quality")
			
			
	if options.trim_masked:
		log("Trim_masked: "+options.trim_masked)
		
		if (options.trim_masked == "yes"):
			cmd_args_array.append("--trim-masked")
		else:
			cmd_args_array.append("--no-trim-masked")
		
	if options.coverage:
		log("Coverage: "+options.coverage)
		
		
		if(options.coverage != "undef"):
			cmd_args_array.append("-c")
			cmd_args_array.append(options.coverage)

	if options.trim_length:
		log("Trim_length: "+options.trim_length)
		
		if(options.trim_length != "undef"):
			cmd_args_array.append("-t")
			cmd_args_array.append(options.trim_length)
		
	if options.trim_quality:
		log("Trim_quality: "+options.trim_quality)
		
		if(options.trim_quality != "undef"):
			cmd_args_array.append("-q")
			cmd_args_array.append(options.trim_quality)
		
	if options.erode:
		log("Erode: "+options.erode)
		
		if(options.erode != "undef"):
			cmd_args_array.append("-e")
			cmd_args_array.append(options.erode)
		
	if options.erode_strand:
		log("Erode_strand: "+options.erode_strand)
		
		if(options.erode_strand != "undef"):
			cmd_args_array.append("-E")
			cmd_args_array.append(options.erode_strand)
		
	if options.cov_hist:
		log("Coverage_histogram File: "+options.cov_hist)
		if(options.cov_hist != "None"):
			cmd_args_array.append("--coverage-hist="+options.cov_hist)
		
	if options.graph:
		log("Dot_graph File: "+options.graph )
		if(options.graph != "None"):
			cmd_args_array.append("-g")
			cmd_args_array.append(options.graph)
		
	if options.snp:
		log("SNP File: "+options.snp )
		if(options.graph != "None"):
			cmd_args_array.append("-s")
			cmd_args_array.append(options.snp)
		
	if options.list_of_file:
		allReadsFile = getListFromFile(options.list_of_file)
		log("Reads file list: "+" ".join(allReadsFile))
	
		for reads_file in allReadsFile:
			cmd_args_array.append(reads_file)
	else:
		print "Error: No list of reads file specified"
		sys.exit()

	# Execution log
	print("\n" + '################################################################' + "\n")
	print(check_output(['ABYSS', '--version']))
	print('################################################################' + "\n")

	print "Abyss assembler will be executed with the following command line: \n"
	print " ".join(cmd_args_array) + "\n"

	# Create output log file
	outFile = os.path.join(os.curdir, logPass)
	outptr = file(outFile, "w")

	# Create error log file
	errFile = os.path.join(os.curdir, "error.log")
	errptr = file(errFile, "w")

	# Call the subprocess using convenience method
	retval = subprocess.call(cmd_args_array, 0, None, None, outptr, errptr)

	# Close log handles
	errptr.close()
	outptr.close()

	# Check the process exit code
	if not retval == 0:
		errptr = file(errFile, "r")
		errData = errptr.read()
		errptr.close()
		print "Error "+errData
		raise Exception("Error executing command: " + repr(errData))

	if (os.path.exists(options.out) and os.path.getsize(options.out) > 0):
		log("Done ...")
	
	print "\nStop time: ", strftime("%d-%m-%Y_%H:%M:%S", localtime())
	print("################################################################")
	print("#                        End of execution                      #")
	print("################################################################\n")