#!/usr/bin/python
# Nom de fichier : calculMeancov.py

from Bio import SeqIO
from time import localtime, strftime
import optparse, os, shutil, subprocess, sys, tempfile

usage = "calculmeancov -i reference -s samfile -o output"

def calculmeancov(input, sam, output):

	length={}
	cover={}
	mappedreads=0

	#get ref sequences lenght
	for seq_record in SeqIO.parse(input, "fasta"):
		length[seq_record.id]=len(seq_record)
		cover[seq_record.id]=0

	#get sam sequences lenght
	samfile = open(sam, 'r')
	for line in samfile:
		t_line = line.split('\t')
		if len(t_line) > 8:
			if t_line[2]!="*":
				cover[t_line[2]]+=len(t_line[9])
				mappedreads+=1
	
	samfile.close()

	#write coverage in output file
	outfile = open(output, "w")
	for key in cover:
		coverage = cover[key] / length[key]
		outfile.write(key + "\t" + str(coverage) + "\n")
	outfile.close()


def __main__():
 
	# Initializations
	version = "1.0"
	
	#Parse Command Line
	parser = optparse.OptionParser()
	parser.add_option( '-i', '--input', dest='input', help='the input ref sequences file name' )
	parser.add_option( '-s', '--sam', dest='sam', help='the input sam file name' )
	parser.add_option( '-o', '--output', dest='out', help='the ouput file name' )
	
	(options, args) = parser.parse_args()
	
	if options.input is None or options.sam is None :
		print usage
		sys.exit()
	
	outputname=options.input+".cov"
	if options.out:
		outputname=options.out
	
	# Welcome message
	print("################################################################")
	print("#         Welcome in calculmeancov  (Version " + version + ")              #")
	print("################################################################")
	print "Start time: ", strftime("%d-%m-%Y_%H:%M:%S", localtime()), "\n"
	
	
	print "input ref file name = " + options.input + "\n"
	print "input sam file name = " + options.sam + "\n"
	print "output file name = " + outputname + "\n"
	
	calculmeancov( str(options.input), str(options.sam), outputname )
	
	print "\nStop time: ", strftime("%d-%m-%Y_%H:%M:%S", localtime())
	print("################################################################")
	print("#                        End of execution                      #")
	print("################################################################\n")
	
	
if __name__=="__main__": __main__()